@@ -4,7 +4,7 @@
 /*								*/
 /* Some Define and Structs					*/
 /****************************************************************/
-/* bgp.h,v 1.32 2001/09/05 11:14:54 kuniaki Exp */
+/* bgp.h,v 1.40 2001/09/27 02:45:40 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2001 Internet Initiative Japan Inc.
@@ -47,7 +47,7 @@
 #include <unistd.h>
 #include "utils.h"
 
-#define BV_VERSION	"0.29Alpha"
+#define BV_VERSION	"0.30Alpha"
 #define BV_COPYRIGHT	"Copyright (c) 1998-2001 Internet Initiative Japan Inc."
 
 /* BGP Header Type field defines */
@@ -107,6 +107,12 @@
 #define SOCK_ACCEPTING		1
 #define SOCK_CONNECTED		2
 
+/* OPEN Option Types */
+#define BGPOPT_AUTHENTICATION		1
+#define BGPOPT_CAPABILITY		2	/* RFC2842 */
+
+/* Capability Code -- RFC2842 */
+
 /* Err Codes */
 #define MESSAGE_HEADER_ERROR		1
 #define OPEN_MESSAGE_ERROR		2
@@ -125,6 +131,7 @@
 #define UNSUPPORTED_OPTIONAL_PARAMETER	4
 #define AUTHENTICATION_FAILURE		5
 #define UNACCEPTABLE_HOLD_TIME		6
+#define UNSUPPORTED_CAPABILITY		7	/* RFC2842 */
 
 #define MALFORMED_ATTRIBUTE_LIST	1
 #define UNRECOGNIZED_WELL_KNOWN_ATTRIBUTE	2
@@ -166,12 +173,15 @@
 #else
 #define FNAMESIZE		1024
 #endif
+#define DESCSIZE		128
 
 /* finish command parameter */
 #define PROC_SHUTDOWN		1
 #define PROC_REBOOT		2
 #define PROC_CHILDEND		3
 
+#define DEFSYSNAME		"BGPView"
+
 typedef u_long	net_ulong;	/* network-byte-ordered u_long */
 
 extern	net_ulong	IDENT;
@@ -184,10 +194,13 @@ typedef struct {
 } BGPV4_HEADER;
 
 typedef struct {
+	char		description[DESCSIZE];	/* Description */
+	boolean		peerstatus;		/* Peer Status (Shutdown) */
 	net_ulong	neighbor;		/* Neighbor Address */
 	u_short		remote_as;		/* REMOTE AS */
 	net_ulong	local_id;		/* local IDENT */
 	u_short		local_as;		/* local AS */
+	char		admin_addr[FNAMESIZE];  /* Peer Admin Email */
 	int		soc;			/* Socket Discripter */
 	int		flag;			/* Socket Flags	*/
 	int		status;			/* BGP Status (FSM) */
@@ -227,6 +240,7 @@ extern char	syslogname[];
 extern int	facility;
 extern char	configfile[];
 extern char	spooldir[];
+extern char	sysname[16];
 
 #ifdef RTHISTORY
 extern struct timeval	rthistcleartime;
