@@ -4,7 +4,7 @@
 /*                                                              */
 /* Main Routine                                                 */
 /****************************************************************/
-/* bgpview.c,v 1.33 2001/08/13 13:38:29 kuniaki Exp */
+/* bgpview.c,v 1.38 2001/09/27 02:45:40 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -75,6 +75,7 @@ int		facility;
 char		configfile[FNAMESIZE];
 struct timeval	uptime;
 char		spooldir[FNAMESIZE];
+char		sysname[16];
 
 #ifdef RTHISTORY
 struct timeval	rthistcleartime;
@@ -132,6 +133,7 @@ void finish(int mode)
 
 void finish_sig()
 {
+	notify_systemnotify("SHUTDOWN", "BGPView received shutdown signal");
 	finish(PROC_SHUTDOWN);
 }
 
@@ -199,6 +201,9 @@ int init()
 	bvs_init_set_default();
 	init_routertt();
 	init_mailannounce();
+
+	strcpy(sysname, DEFSYSNAME);
+
 #ifdef RADIX
 	init_radix();
 #endif
@@ -459,7 +464,10 @@ int len;
 				continue;
 			}
 			perror("select");
-			exit(1);
+			notify_systemnotify("REBOOT",
+				"Select Error : sockwrite");
+			finish(PROC_REBOOT);
+			return(0);
 		}
 		if (FD_ISSET(soc, &wset)) {
 			written = write(soc, ptr, left);
@@ -576,35 +584,91 @@ char sc;
 	return sz;
 }
 
-int recvdata(pn,len)
-int pn;
-int len;
+#ifdef DVERV
+void recv_dump(char *data, int len, int pn)
+{
+	char	buf[32];
+	int 	length;
+	char	*ptn, *pt;
+	int	cnt;
+	struct tm	*tm;
+	time_t		clock;
+
+	length = len;
+	ptn = data;
+	pt = ptn+length;
+	cnt = 0;
+	sprintf(logstr, "RECV DUMP LEN  : %d Octets\n", length);
+	pr_log(VLOG_GLOB, pn, logstr, OFF);
+	time(&clock);
+	tm = localtime(&clock);
+	strftime(buf, 30, "%Y/%m/%d %H:%M:%S", tm);
+	sprintf(logstr, "RECV DUMP DATE : %s\n", buf);
+	pr_log(VLOG_GLOB, pn, logstr, OFF);
+	pr_log(VLOG_GLOB, pn, "RECV DUMP 0000 : ", OFF);
+	while(ptn < pt) {
+		sprintf(logstr, "%08X ", buf2ulong(ptn));
+		pr_log(VLOG_GLOB, pn, logstr, OFF);
+		ptn += 4;
+		cnt++;
+		if ((cnt % 4) == 0) {
+			sprintf(logstr, "\nRECV DUMP %04d : ", cnt*4);
+			pr_log(VLOG_GLOB, pn, logstr, OFF);
+		}
+	}
+	pr_log(VLOG_GLOB, pn, "\n", OFF);
+	return;
+}
+#endif
+
+
+int recvdata(int pn,int len)
 {
 	int	ret;
+	int	left;
+	int	rcnt;
+	int	readed;
+	char	readbuf[BGP_MAX_PACKET_SIZE];
 
 #ifdef DEBUG
 	printf("Receiving Data : %d Octets\n",len);
 #endif
 	if (len <= 0) return 0;
 
+	left = len;
 	if (peer[pn].data != NULL) free(peer[pn].data);
 	peer[pn].data = (char *)malloc(len);
+	bzero(peer[pn].data, len);
 #ifdef DEBUG
 	printf("Malloc OK  : %d Octets\n",len);
 #endif
-	ret = read(peer[pn].soc, (char *)peer[pn].data, len); 
+	rcnt = 0;
+	readed = 0;
+	while(1) {
+		bzero(&readbuf[0], BGP_MAX_PACKET_SIZE);
+		ret = read(peer[pn].soc, &readbuf[0], left); 
+		memcpy(((char *)peer[pn].data + readed), readbuf, ret);
 #ifdef DEBUG
-	printf("Received Data  : %d Octets\n",ret);
+		printf("Received Data  : %d Octets\n",ret);
 #endif
+		left -= ret;
+		readed += ret;
+		if (left <= 0) break;
+		rcnt++;
+		if (rcnt >= 6) break;
+	}
 		
-	if ((len != ret) && (ret != 0)) {
+	if ((left > 0) && (ret != 0)) {
 		sprintf(logstr, "Data Read Fail : %d/%d : peer = %d\n",
-			ret, len, pn);
+			readed, len, pn);
 		pr_log(VLOG_GLOB, pn, logstr, ON);
 		ret = -1;
 	}
-	peer[pn].datalen = ret;
-	return ret;
+	peer[pn].datalen = readed;
+#ifdef DVERV
+	recv_dump(peer[pn].data, peer[pn].datalen, pn);
+#endif
+	return readed;
 }
 
 int check_header(pn)
@@ -850,7 +914,11 @@ main(int argc, char *argv[])
 				printf("QUIT FSM Code = %d\n",quit);
 #endif
 				peer[pn].soc = -1;
-				peer[pn].event = BGP_START;
+				if (peer[pn].peerstatus == false) {
+					peer[pn].event = BGP_NONE;
+				} else {
+					peer[pn].event = BGP_START;
+				}
 				peer[pn].status = IDLE;
 				quit = 0;
 			}
@@ -904,7 +972,10 @@ main(int argc, char *argv[])
 					continue;
 				}
 				perror("select");
-				exit(1);
+				notify_systemnotify("REBOOT",
+						"Select Error : main() - 1");
+				finish(PROC_REBOOT);
+				return(1);
 			}
 
 			/* Time Out Check */
@@ -957,13 +1028,21 @@ main(int argc, char *argv[])
 				pr_log(VLOG_GLOB, pn, 
 					"Re-Initial Timer Expired\n", ON);
 				peer[pn].resume = OFF;
-				peer[pn].event = BGP_START;
+				if (peer[pn].peerstatus == false) {
+					peer[pn].event = BGP_NONE;
+				} else {
+					peer[pn].event = BGP_START;
+				}
 				peer[pn].status = IDLE;
 				break;
 			case ID_BGPINFO_TIMER:
 				pr_log(VLOG_GLOB, pn, 
 					"BGP_INFO Timer Expired\n", ON);
 				write_info(pn);
+				if (bgpinfo[pn].interval == 0) {
+					bgpinfo[pn].infostatus == false;
+				}
+				if (bgpinfo[pn].infostatus == false) break;
 				timer_value.tv_sec = bgpinfo[pn].interval;
 				timer_value.tv_usec = 0;
 				if (!entry_timer(&timer_value,
@@ -971,7 +1050,10 @@ main(int argc, char *argv[])
 						ID_BGPINFO_TIMER)) {
 					pr_log(VLOG_GLOB, LOG_ALL,
                                 	"failed to allocate memory.\n", ON);
-					exit(1);
+					notify_systemnotify("REBOOT",
+						"Memory Allocation Error : main() - 2");
+					finish(PROC_REBOOT);
+					return(1);
 				}
 				break;
 			default:
@@ -1015,7 +1097,11 @@ main(int argc, char *argv[])
 						"  Socket Clear\n", OFF);
 					close(ns);
 				} else {
-					if (peer[pn].resume == ON) {
+					if (peer[pn].peerstatus == false) {
+						pr_log(VLOG_GLOB, LOG_ALL,
+						  "  Disabled Peer\n", OFF);
+						close(ns);
+					} else if (peer[pn].resume == ON) {
 						pr_log(VLOG_GLOB, LOG_ALL,
 						  "  Now Resuming Peer\n", OFF);
 						close(ns);
