@@ -3,7 +3,7 @@
 /*                                                              */
 /* BGPView Shell Commands                                       */
 /****************************************************************/
-/* bvcommand.c,v 1.45 2001/08/15 07:15:35 kuniaki Exp */
+/* bvcommand.c,v 1.52 2001/09/27 02:45:40 kuniaki Exp */
 
 /*
  * Copyright (c) 2000 - 2001 Internet Initiative Japan Inc.
@@ -60,12 +60,14 @@
 /****** Functions **********/
 int bvc_shutdown(BVS_CMD_PARAM *prm)
 {
+	notify_systemnotify("SHUTDOWN", "Shutdown by BGPView Shell Command");
 	finish(PROC_SHUTDOWN);
 	return(0);
 }
 
 int bvc_reboot(BVS_CMD_PARAM *prm)
 {
+	notify_systemnotify("REBOOT", "Reboot by BGPView Shell Command");
 	finish(PROC_REBOOT);
 	return(0);
 }
@@ -171,6 +173,178 @@ long ptmtosec(char *str, long *tvsec, long *range)
 	return(*tvsec);
 }
 
+int bvc_setannouncestatus(BVS_CMD_PARAM *prm)
+{
+	int		clno;
+	u_long		adrs;
+	boolean		action;
+	int		pn;
+
+	clno = (prm->adrs_boolean).clno;
+	adrs = (prm->adrs_boolean).addr;
+	action = (prm->adrs_boolean).swc;
+
+	pn = check_id1topn(adrs);
+	if (pn < 0) {
+		bvs_write(clno, "Neighbor Address is not configured\r\n");
+		return(BVS_CNT_END);
+	}
+	if (action == 1) {
+		if (peer[pn].status_notify == true) {
+			bvs_write(clno, "Notifying peer status already ON\r\n");
+		} else {
+			peer[pn].status_notify = true;
+			bvs_write(clno, "Notifying peer status changed to ON\r\n");
+		}
+	} else {
+		if (peer[pn].status_notify == false) {
+			bvs_write(clno, "Notifying peer status already OFF\r\n");
+		} else {
+			peer[pn].status_notify = false;
+			bvs_write(clno, "Notifying peer status changed to OFF\r\n");
+		}
+	}
+	return(BVS_CNT_END);
+}
+
+int bvc_setpeerdisable(BVS_CMD_PARAM *prm)
+{
+	int		clno;
+	u_long		adrs;
+	int		pn;
+	char		prline[128];
+	struct in_addr  sin;
+
+
+	clno = (prm->adrs).clno;
+	adrs = (prm->adrs).addr;
+	sin.s_addr = adrs;
+
+	pn = check_id1topn(adrs);
+	if (pn < 0) {
+		sprintf(prline, "Neighbor %s was not configured\r\n",
+							inet_ntoa(sin));
+		bvs_write(clno, prline);
+		return(BVS_CNT_END);
+	}
+
+	if (peer[pn].peerstatus == false) {
+		sprintf(prline, "Neighbor %s already disabled.\r\n",
+							inet_ntoa(sin));
+		bvs_write(clno, prline);
+		return(BVS_CNT_END);
+	}
+	peer[pn].peerstatus = false;
+	bgpinfo[pn].infostatus = false;
+	clear_peer(pn);
+
+	sprintf(prline, "Disable neighbor %s\r\n", inet_ntoa(sin));
+	bvs_write(clno, prline);
+
+	return(BVS_CNT_END);
+}
+
+int bvc_setpeerenable(BVS_CMD_PARAM *prm)
+{
+	int		clno;
+	u_long		adrs;
+	int		pn;
+	char		prline[128];
+	struct in_addr	sin;
+	struct timeval	timer_value;
+
+	clno = (prm->adrs).clno;
+	adrs = (prm->adrs).addr;
+	sin.s_addr = adrs;
+
+	pn = check_id1topn(adrs);
+	if (pn < 0) {
+		sprintf(prline, "Neighbor %s was not configured\r\n",
+							inet_ntoa(sin));
+		bvs_write(clno, prline);
+		return(BVS_CNT_END);
+	}
+
+	if (peer[pn].peerstatus == true) {
+		sprintf(prline, "Neighbor %s already enabled.\r\n",
+							inet_ntoa(sin));
+		bvs_write(clno, prline);
+		return(BVS_CNT_END);
+	}
+	peer[pn].peerstatus = true;
+	peer[pn].event = BGP_NONE;
+	peer[pn].status = IDLE;
+	bgpinfo[pn].infostatus = true;
+	recount_info(pn);
+	timer_value.tv_sec = bgpinfo[pn].interval;
+	timer_value.tv_usec = 0;
+	if (!entry_timer(&timer_value, peer[pn].neighbor, ID_BGPINFO_TIMER)) {
+		pr_log(VLOG_GLOB, LOG_ALL, "failed to allocate memory.\n", ON);
+		notify_systemnotify("REBOOT",
+			"Memory Allocation Error : bvc_setinfologstatus()");
+		finish(PROC_REBOOT);
+		return(BVS_CNT_END);
+	}
+
+	sprintf(prline, "Enable neighbor %s\r\n", inet_ntoa(sin));
+	bvs_write(clno, prline);
+
+	return(BVS_CNT_END);
+}
+
+int bvc_setinfologstatus(BVS_CMD_PARAM *prm)
+{
+	int		clno;
+	u_long		adrs;
+	boolean		sw;
+	char		prline[128];
+	int		pn;
+	struct in_addr	sin;
+	struct timeval	timer_value;
+
+	clno = (prm->adrs_boolean).clno;
+	adrs = (prm->adrs_boolean).addr;
+	sw   = (prm->adrs_boolean).swc;
+	sin.s_addr = adrs;
+
+	pn = check_id1topn(adrs);
+	if (pn < 0) {
+		sprintf(prline, "Neighbor %s was not configured\r\n",
+						inet_ntoa(sin));
+		bvs_write(clno, prline);
+		return(BVS_CNT_END);
+	}
+
+	if (bgpinfo[pn].infostatus == false) {
+		if (sw == false) {
+			bvs_write(clno, "Infomation log already disabled.\r\n");
+		} else {
+			bgpinfo[pn].infostatus = true;
+			recount_info(pn);
+			timer_value.tv_sec = bgpinfo[pn].interval;
+			timer_value.tv_usec = 0;
+			if (!entry_timer(&timer_value, peer[pn].neighbor,
+							ID_BGPINFO_TIMER)) {
+				pr_log(VLOG_GLOB, LOG_ALL,
+					"failed to allocate memory.\n", ON);
+				notify_systemnotify("REBOOT",
+				"Memory Allocation Error : bvc_setinfologstatus()");
+				finish(PROC_REBOOT);
+				return(BVS_CNT_END);
+			}
+			bvs_write(clno, "Infomation log was set enable.\r\n");
+		}
+	} else {
+		if (sw == true) {
+			bvs_write(clno, "Infomation log already enabled.\r\n");
+		} else {
+			bgpinfo[pn].infostatus = false;
+			bvs_write(clno, "Infomation log was set disable.\r\n");
+		}
+	}
+	return(BVS_CNT_END);
+}
+
 int bvc_bgpsum(BVS_CMD_PARAM *prm)
 {
 	int		clno;
@@ -183,9 +357,9 @@ int bvc_bgpsum(BVS_CMD_PARAM *prm)
 	clno = (prm->noparam).clno;
 
 	bvs_write(clno, "\r\nRoute Summary---\r\n");
-	sprintf(prline, " Received Prefixes : %ld\r\n", radix_prefixes());
+	sprintf(prline, " Received Routes   : %ld\r\n", radix_prefixes());
 	bvs_write(clno, prline);
-	sprintf(prline, " Received Routes   : %ld\r\n", radix_routes());
+	sprintf(prline, " Received Prefixes : %ld\r\n", radix_routes());
 	bvs_write(clno, prline);
 	sprintf(prline, " RADIX Tree Nodes  : %ld\r\n", radix_nodes());
 	bvs_write(clno, prline);
@@ -195,10 +369,16 @@ int bvc_bgpsum(BVS_CMD_PARAM *prm)
 		adr.s_addr = peer[pn].local_id;
 		strcpy(localident, inet_ntoa(adr));
 		adr.s_addr = peer[pn].neighbor;
-		sprintf(prline, "%2d %-15s %-5d %-15s %-5d %-12s\r\n",
-			pn, localident, peer[pn].local_as,
-			inet_ntoa(adr), peer[pn].remote_as,
-			fsm_state_names[peer[pn].status]);
+		if (peer[pn].peerstatus == true) {
+			sprintf(prline, "%2d %-15s %-5d %-15s %-5d %-12s\r\n",
+				pn, localident, peer[pn].local_as,
+				inet_ntoa(adr), peer[pn].remote_as,
+				fsm_state_names[peer[pn].status]);
+		} else {
+			sprintf(prline, "%2d %-15s %-5d %-15s %-5d Admin Down\r\n",
+				pn, localident, peer[pn].local_as,
+				inet_ntoa(adr), peer[pn].remote_as);
+		}
 		bvs_write(clno, prline);
 	}
 	return(BVS_CNT_END);
@@ -277,6 +457,9 @@ int bvc_shoneighbor(BVS_CMD_PARAM *prm)
 		bvs_write(clno, "Not found peer\r\n");
 	} else {
 		bvs_write(clno, "\r\nBGP Peer Information:\r\n");
+		sprintf(prline, "    Peer Name        : %s\r\n",
+					peer[pn].description);
+		bvs_write(clno, prline);
 		sprintf(prline, "    Neighbor Address : %s\r\n",
 					inet_ntoa(sin));
 		bvs_write(clno, prline);
@@ -290,8 +473,16 @@ int bvc_shoneighbor(BVS_CMD_PARAM *prm)
 		sprintf(prline, "    Individual AS    : %d\r\n",
 					peer[pn].local_as);
 		bvs_write(clno, prline);
-		sprintf(prline, "    BGP Peer Status  : %s\r\n",
+		if (peer[pn].peerstatus == false) {
+			bvs_write(clno, 
+				"    BGP Peer Status  : Admin Down\r\n");
+		} else {
+			sprintf(prline, "    BGP Peer Status  : %s\r\n",
 					fsm_state_names[peer[pn].status]);
+			bvs_write(clno, prline);
+		}
+		sprintf(prline, "    Admin Address    : %s\r\n",
+					peer[pn].admin_addr);
 		bvs_write(clno, prline);
 		sprintf(prline, "    Hold Timer       : %d sec\r\n",
 					peer[pn].holdtimer);
@@ -305,6 +496,12 @@ int bvc_shoneighbor(BVS_CMD_PARAM *prm)
 		sprintf(prline, "    Prefix Logfile   : %s\r\n",
 					peer[pn].prefixout);
 		bvs_write(clno, prline);
+		sprintf(prline, "    Notifying Status : %s\r\n",
+						BOOL[peer[pn].status_notify]);
+		bvs_write(clno, prline);
+		sprintf(prline, "    INFO Log Status  : %s\r\n",
+						BOOL[bgpinfo[pn].infostatus]);
+		bvs_write(clno, prline);
 		bvs_write(clno, "    Log Type:\r\n");
 
 		sprintf(prline, "        Global Log               : %s\r\n",
@@ -375,7 +572,10 @@ int bvc_shoproc(BVS_CMD_PARAM *prm)
 
 		execl(TOPCMD, "top", "-b", (char *)NULL );
 		perror("top command exec error");
-		exit(1);
+		pr_log(VLOG_GLOB, LOG_ALL, 
+				"top command exec error\n", ON);
+		bvs_write(clno, "top command exec error\r\n");
+		return(BVS_CNT_END);
 	}
 
 	close(wri[0]);
@@ -862,7 +1062,10 @@ int pr_routesum(int clno, R_LIST *rlist, net_ulong neighbor)
 		strcat(prline, prline2);
 		sprintf(prline2, "%-8s ", sectoptm(timediff, tmstr));
 		strcat(prline, prline2);
-		strcat(prline, rinfo->path_set);
+		/* strcat(prline, rinfo->path_set); */
+		if (rinfo->pathset_ptr != NULL) {
+			strcat(prline, rinfo->pathset_ptr);
+		}
 		if (ATRFL_ISSET(rinfo->attribute_flag, SET_ORIGIN)) {
 			switch(rinfo->origin_type) {
 			case ORIGIN_IGP:
@@ -929,7 +1132,7 @@ int bvc_show_bgproute_all(BVS_CMD_PARAM *prm)
 	clno = (prm->noparam).clno;
 
 	if (*(cnode+clno) == NULL) {
-		bvs_write(clno, "\r\n ! : Incltive Route\r\n\r\n");
+		bvs_write(clno, "\r\n ! : Inactive Route\r\n\r\n");
 		bvs_write(clno, 
 "   Prefix           Next Hop         Metric Loc_Pef Time    AS_Path\r\n");
 	}
@@ -1078,7 +1281,10 @@ int output_routesum(R_LIST *rlist, net_ulong neighbor, FILE *ofp)
 		strcat(prline, prline2);
 		sprintf(prline2, "%-8s ", sectoptm(timediff, tmstr));
 		strcat(prline, prline2);
-		strcat(prline, rinfo->path_set);
+		/* strcat(prline, rinfo->path_set); */
+		if (rinfo->pathset_ptr != NULL) {
+			strcat(prline, rinfo->pathset_ptr);
+		}
 		if (ATRFL_ISSET(rinfo->attribute_flag, SET_ORIGIN)) {
 			switch(rinfo->origin_type) {
 			case ORIGIN_IGP:
@@ -1356,7 +1562,7 @@ int bvc_show_bgproute_neighbor(BVS_CMD_PARAM *prm)
 	neigh = (prm->adrs).addr;
 
 	if (*(cnode+clno) == NULL) {
-		bvs_write(clno, "\r\n ! : Incltive Route\r\n\r\n");
+		bvs_write(clno, "\r\n ! : Inactive Route\r\n\r\n");
 		bvs_write(clno, 
 "   Prefix           Next Hop         Metric Loc_Pef Time    AS_Path\r\n");
 	}
@@ -1494,7 +1700,10 @@ int pr_routetime(int clno, R_LIST *rlist, long tsec, long range, int mode)
 		strcat(prline, prline2);
 		sprintf(prline2, "%-8s ", sectoptm(timediff, tmstr));
 		strcat(prline, prline2);
-		strcat(prline, rinfo->path_set);
+		/* strcat(prline, rinfo->path_set); */
+		if (rinfo->pathset_ptr != NULL) {
+			strcat(prline, rinfo->pathset_ptr);
+		}
 		if (ATRFL_ISSET(rinfo->attribute_flag, SET_ORIGIN)) {
 			switch(rinfo->origin_type) {
 			case ORIGIN_IGP:
@@ -1746,7 +1955,12 @@ int rtcount(R_LIST *rlist, u_long *aslen, u_long *ascnt,
 
 		msklen[rinfo->length-1]++;
 		(*ascnt)++;
-		strcpy(aspathes, rinfo->path_set);
+		/* strcpy(aspathes, rinfo->path_set); */
+		if (rinfo->pathset_ptr != NULL) {
+			strcpy(aspathes, rinfo->pathset_ptr);
+		} else {
+			strcpy(aspathes, "");
+		}
 		pathp = strtok(aspathes, " ");
 		while(pathp != NULL) {
 #ifdef DEBUG
@@ -1917,7 +2131,14 @@ int bvc_clear_route_history(BVS_CMD_PARAM *prm)
 			common_off_timer(rtclear_id);
 			rtclear_id = common_entry_timer(&rthistcleartime, 
 							0, rthisttimerclear);
-			if (rtclear_id == -1) exit(1);
+			if (rtclear_id == -1) {
+				pr_log(VLOG_GLOB, LOG_ALL,
+				 "Route History Clearing Timer Set Fail\n", ON);
+				notify_systemnotify("REBOOT",
+				       "Route History Clearing Timer Set Fail");
+				finish(PROC_REBOOT);
+				return(1);
+			}
 
 			return(BVS_CNT_FINISH);
 		}
@@ -2315,6 +2536,6 @@ void testfunc(int id, int prm)
 
 	sprintf(prline, "Common Timer Expierd : ID=%d\r\n", id);
 	bvs_write(prm, prline);
-	bvs_write(prm, BVS_PROMPT);
+	bvs_write(prm, bvs_prompt);
 }
 #endif
