@@ -4,7 +4,7 @@
 /*                                                              */
 /* BGPView Shell 			                        */
 /****************************************************************/
-/* bvshell.c,v 1.51 2001/09/05 11:14:55 kuniaki Exp */
+/* bvshell.c,v 1.56 2001/09/27 02:45:41 kuniaki Exp */
 
 /*
  * Copyright (c) 2000 Internet Initiative Japan Inc.
@@ -61,6 +61,9 @@ BVS_CLIENT	*bvs_clients;
 
 BVS_CMDS	*cmds;
 
+char		bvs_prompt[32];
+char 		bvs_promptmark[] = BVSPROMPTMKLIST;
+
 /***** Subroutins *****/
 
 int bvs_cmd_exit(BVS_CMD_PARAM *prm)
@@ -183,6 +186,18 @@ void bvs_init_set_default()
 						"Clear BGP Session", false);
 	bvs_entry_command(bvc_shutdown, "shutdown", "shutdown BGPView", false);
 	bvs_entry_command(bvc_reboot, "reboot", "Reboot BGPView", false);
+	bvs_entry_command(bvc_setannouncestatus, 
+				"set neighbor %ADR announce status %SWC",
+				"Enabling/Disabling status change announcement",
+				false);
+	bvs_entry_command(bvc_setpeerdisable, "set neighbor %ADR disable",
+				"Disable peer", false);
+	bvs_entry_command(bvc_setpeerenable, "set neighbor %ADR enable",
+				"Enable peer", false);
+	bvs_entry_command(bvc_setinfologstatus, 
+				"set neighbor %ADR infolog %SWC",
+				"Enabling/Disabling Infomation Log Status",
+				false);
 	bvs_entry_command(bvc_bgpsum, "show ip bgp summary", 
 					"Show BGP Peer Summary", false);
 	bvs_entry_command(bvc_shousers, "show users", 
@@ -661,6 +676,22 @@ void *bvs_search_function(const char *cmdline, BVS_CMD_PARAM *param, boolean *mo
 				    strcpy((char *)prmptr, strbuf);
 				    prmptr += 128;
 				    lv = intbuf;
+				} else if (strcmp((curptr->word)+1,"SWC")==0) {
+				    strcpy(strbuf, words[lv]);
+				    intbuf = 0;
+				    while(strbuf[intbuf] != '\0') {
+				    	strbuf[intbuf] = toupper(strbuf[intbuf]);
+				    	intbuf++;
+				    }
+				    if (strcmp(strbuf, "ON") == 0) {
+				    	*(char *)prmptr = (char)1;
+				    } else if (strcmp(strbuf, "OFF") == 0) {
+				    	*(char *)prmptr = (char)0;
+				    } else {
+				    	/* Parameter Error */
+				    	return(NULL);
+				    }
+				    prmptr += 1;
 				}
 		  	    } else {
 				return(NULL);
@@ -705,7 +736,8 @@ int bvs_exec_command(int clno)
 
 	ret = 0;
 	if ((bvs_clients+clno)->status == BVSC_LOGIN) {
-		if (strcmp((bvs_clients+clno)->cmdline, bvs_pass) == 0) {
+		if (checkpw(bvs_pass, (bvs_clients+clno)->cmdline) == 1) {
+		/* if (strcmp((bvs_clients+clno)->cmdline, bvs_pass) == 0) { */
 			(bvs_clients+clno)->status = BVSC_INPUT_CMD;
 			sin.s_addr = htonl((bvs_clients+clno)->addr);
 			sprintf(logstr, "BVS Logged in from %s\n",
@@ -796,7 +828,7 @@ int bvs_exec_command(int clno)
 		bvs_write(clno, "\r\nPassword:");
 	} else {
 		if ((ret == 0)||(ret == 2)) {
-			bvs_write(clno, BVS_PROMPT);
+			bvs_write(clno, bvs_prompt);
 		}
 	}
 
@@ -1090,6 +1122,10 @@ int bvs_cmd_pr_help(int clno, const char *cmdline)
 					    if (strcmp(ptrlist[nextw]->word,
 					    			"%LSR") == 0) {
 					    	strcpy(swword, "<Words>");
+					    } else
+					    if (strcmp(ptrlist[nextw]->word,
+					    			"%SWC") == 0) {
+					    	strcpy(swword, "<ON/OFF>");
 					    }
 					} else {
 						strcpy(swword, ptrlist[nextw]->word);
@@ -1351,7 +1387,7 @@ int clearscreen(int clno)
 	cls[8] = '0';
 	cls[9] = 'H';
 	cls[10] = '\0';
-	strcat(cls, BVS_PROMPT);
+	strcat(cls, bvs_prompt);
 	strcat(cls, (bvs_clients+clno)->cmdline);
 
 	bvs_write(clno, cls);
@@ -1374,7 +1410,7 @@ int bvs_cont_proc()
 	(*((bvs_clients+clno)->cont_func))(&((bvs_clients+clno)->cont_param));
 			if ((ret == 0)||(ret == 2)) {
 				(bvs_clients+clno)->cont_code = 0;
-				bvs_write(clno, BVS_PROMPT);
+				bvs_write(clno, bvs_prompt);
 			}
 		}
 		if (ret == 1) retc = 1;
@@ -1500,7 +1536,7 @@ int bvs_input(fd_set *fds)
 					complen = bvs_cmd_pr_help(cnt,
 						(bvs_clients+cnt)->cmdline);
 					if (complen > 0) {
-						strcpy(buf, BVS_PROMPT);
+						strcpy(buf, bvs_prompt);
 						strcat(buf, 
 						    (bvs_clients+cnt)->cmdline);
 						bvs_write(cnt, buf);
@@ -1528,7 +1564,7 @@ int bvs_input(fd_set *fds)
 					if (strlen((bvs_clients+cnt)->cmdline) >
 							(bvs_clients+cnt)->col) {
 						strcpy(buf, "\r\n");
-						strcat(buf, BVS_PROMPT);
+						strcat(buf, bvs_prompt);
 						strcat(buf, 
 						    (bvs_clients+cnt)->cmdline);
 						bvs_write(cnt, buf);
@@ -1637,6 +1673,13 @@ int bvs_input(fd_set *fds)
 					ins_char(cnt, NULL, 1, BVSE_DEL);
 					cpnt++;
 					break;
+				case 0x05:
+					/* Move Cur End (Ctrl-e) */
+					mov_cur(cnt, 1, 
+					  (strlen((bvs_clients+cnt)->cmdline))
+					  -((bvs_clients+cnt)->col));
+					cpnt++;
+					break;
 				case 0x0B:
 					/* Clear from cursor (Ctrl-k) */
 					ins_char(cnt, NULL,
@@ -1730,6 +1773,9 @@ int bvs_open_new_shell(fd_set *fds)
 						(struct timezone *)0);
 			(bvs_clients + cnt)->col = 0;
 			(bvs_clients + cnt)->rowsize = bvs_rowsize;
+			(bvs_clients + cnt)->userlevel = DEFUSERLEVEL;
+			sprintf(bvs_prompt, "%s%c ", sysname, 
+			     bvs_promptmark[(bvs_clients + cnt)->userlevel]);
 		} else {
 #ifdef DEBUG
 			printf("DEBUG: BVS: Client Socket Maximum\n");
