@@ -4,7 +4,7 @@
 /*                                                              */
 /* BGPView Shell / Defines and Structs                          */
 /****************************************************************/
-/* bvshell.h,v 1.17 2001/09/05 11:14:55 kuniaki Exp */
+/* bvshell.h,v 1.18 2001/09/27 02:45:41 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -61,7 +61,12 @@
 
 #define MAXIMUM_CMD_LENGTH	128
 
-#define BVS_PROMPT		"BGPView# "
+/* #define BVS_PROMPT		"BGPView# " */
+#define BVSUSER_USER		0
+#define BVSUSER_ADMIN		1
+#define BVSPROMPTMKLIST		"%#"
+#define DEFUSERLEVEL		BVSUSER_ADMIN
+
 #define DEF_ROWSIZE		24
 
 /* Client Status */
@@ -99,6 +104,7 @@ typedef struct {
 	boolean		cmd_lock; /* Command input Lock */
 	char		cmdline[MAXIMUM_CMD_LENGTH+1];
 	char		history[BVSC_MAXHISTORY][MAXIMUM_CMD_LENGTH+1];
+	char		userlevel;
 	int		curhitory;
 	int		histnum;
 	int		col;
@@ -121,6 +127,7 @@ typedef struct {
 /* Valiables */
 extern int		bvs_maxcon;
 extern BVS_CLIENT	*bvs_clients;
+extern char		bvs_prompt[32];
 
 /* Functions */
 extern void bvs_init_set_default();
