@@ -4,7 +4,7 @@
 /*                                                              */
 /* Radix Tree Routine                                           */
 /****************************************************************/
-/* radix.c,v 1.18 2001/04/19 11:42:57 kuniaki Exp */
+/* radix.c,v 1.22 2001/09/19 10:27:54 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -81,6 +81,8 @@ int entry_route(RADIX_T *node, ROUTE_INFO *rinfo)
 #ifdef RTHISTORY
 	RT_HISTORY	*histbuf, *histendbuf, *hhist;
 	boolean		actbuf;
+	char            pathsetbuf[4096];
+	ROUTE_INFO	*pathbuf;
 #endif
 
 #ifdef DEBUG
@@ -98,6 +100,8 @@ int entry_route(RADIX_T *node, ROUTE_INFO *rinfo)
 		if (rlist == NULL) {
 			lbuf = (R_LIST *)malloc(sizeof(R_LIST));
 			if (lbuf == NULL) {
+				pr_log(VLOG_GLOB, LOG_ALL, 
+				   "entry_route(lbuf): malloc error\n", ON);
 				free(ribuf);
 				return(0);
 			}
@@ -121,6 +125,11 @@ int entry_route(RADIX_T *node, ROUTE_INFO *rinfo)
 			actbuf = (rlist->route_data)->active;
 			tmvtemp.tv_sec = (rlist->route_data)->firstupdate.tv_sec;
 #endif
+			if (((rlist->route_data)->pathset_ptr) != NULL) {
+				pathbuf = rlist->route_data;
+				free(pathbuf->pathset_ptr);
+				pathbuf->pathset_ptr = NULL;
+			}
 			free(rlist->route_data);
 			rlist->route_data = ribuf;
 #ifdef RTHISTORY
@@ -291,6 +300,7 @@ int withdraw_route(int pn, net_ulong prefix, int masklen)
 	RADIX_T		*node, *bnode;
 	int		rcode;
 	R_LIST		*rlist, *brlist;
+	ROUTE_INFO	*pathbuf;
 #ifdef RTHISTORY
 	RT_HISTORY	*hhist;
 #endif
@@ -299,147 +309,150 @@ int withdraw_route(int pn, net_ulong prefix, int masklen)
 
 	rcode = 0;
 	while(1) {
-		if (node->check_bit > masklen-1) {
-			rcode = 1; /* no route */
-			break;
-		}
+	  if (node->check_bit > masklen-1) {
+	    rcode = 1; /* no route */
+	    break;
+	  }
 #ifdef DEBUG
-		printf("DEBUG: RADIX: DELETE: Checking pref=%08X / mask=%d",
+	  printf("DEBUG: RADIX: DELETE: Checking pref=%08X / mask=%d",
 					ntohl(node->key), node->masklen);
-		printf(" / Check_bit=%d\n",
-						node->check_bit);
+	  printf(" / Check_bit=%d\n", node->check_bit);
 #endif
-		if ((node->key == prefix) && (node->masklen == masklen-1)) {
-			rlist = node->rinfo;
-			if (rlist == NULL) {
-				rcode = 1;
-				break;
-			}
-			brlist = NULL;
-			rcode = 1;
-			while(rlist != NULL) {
-				if (pn == 
-				    check_id1topn(rlist->route_data->source)) {
-					/* delete route */
+	  if ((node->key == prefix) && (node->masklen == masklen-1)) {
+	    rlist = node->rinfo;
+	    if (rlist == NULL) {
+	      rcode = 1;
+	      break;
+	    }
+	    brlist = NULL;
+	    rcode = 1;
+	    while(rlist != NULL) {
+	      if (pn == check_id1topn(rlist->route_data->source)) {
+	      /* delete route */
 #ifdef DEBUG
-	printf("DEBUG: RADIX: DELETE: pref = %08X / Mask = %d",
-		ntohl(rlist->route_data->prefix), rlist->route_data->length);
-	printf(" / Neighbor = %08X\n",
-		ntohl(rlist->route_data->source));
+	        printf("DEBUG: RADIX: DELETE: pref = %08X / Mask = %d",
+	  	  ntohl(rlist->route_data->prefix), rlist->route_data->length);
+	        printf(" / Neighbor = %08X\n", ntohl(rlist->route_data->source));
 #endif
 
 #ifdef RTHISTORY
-					if (deltype == DFLAG) {
-					  hhist = (RT_HISTORY *)malloc
-					  	             (sizeof(RT_HISTORY));
-					  if (hhist != NULL) {
-					    bzero(hhist, sizeof(RT_HISTORY));
-					    (rlist->route_data)->active = false;
-					    hhist->type = WITHDRAW;
-					    gettimeofday(&(hhist->update),
-					    		 (struct timezone *)0);
-					    if ((rlist->route_data)->history == NULL) {
-					      (rlist->route_data)->history = hhist;
-					      (rlist->route_data)->histend = hhist;
-					    } else {
-					      hhist->before = 
-					    	    (rlist->route_data)->histend;
-					      (rlist->route_data)->histend = hhist;
-					      (hhist->before)->next = hhist;
-					    }
-					    rcode = 0;
-					    break;
-					  } else {
-					    free(rlist->route_data);
-					    if (brlist == NULL) {
-						    node->rinfo = rlist->next;
-						    free(rlist);
-						    rlist = node->rinfo;
-					    } else {
-						    brlist->next = rlist->next;
-						    free(rlist);
-						    rlist = brlist->next;
-					    }
-					  }
-					} else {
+	        if (deltype == DFLAG) {
+		  hhist = (RT_HISTORY *)malloc (sizeof(RT_HISTORY));
+	  	  if (hhist != NULL) {
+		    bzero(hhist, sizeof(RT_HISTORY));
+		    (rlist->route_data)->active = false;
+		    hhist->type = WITHDRAW;
+		    gettimeofday(&(hhist->update), (struct timezone *)0);
+		    if ((rlist->route_data)->history == NULL) {
+		      (rlist->route_data)->history = hhist;
+		      (rlist->route_data)->histend = hhist;
+		    } else {
+		      hhist->before = (rlist->route_data)->histend;
+		      (rlist->route_data)->histend = hhist;
+		      (hhist->before)->next = hhist;
+		    }
+		    rcode = 0;
+		    break;
+		  } else {
+		    if (((rlist->route_data)->pathset_ptr) != NULL) {
+		      pathbuf = rlist->route_data;
+		      free(pathbuf->pathset_ptr);
+		      pathbuf->pathset_ptr = NULL;
+		    }
+		    free(rlist->route_data);
+		    if (brlist == NULL) {
+		      node->rinfo = rlist->next;
+		      free(rlist);
+		      rlist = node->rinfo;
+		    } else {
+		      brlist->next = rlist->next;
+		      free(rlist);
+		      rlist = brlist->next;
+		    }
+		  }
+	        } else {
 #endif
-					free(rlist->route_data);
-					if (brlist == NULL) {
-						node->rinfo = rlist->next;
-						free(rlist);
-						rlist = node->rinfo;
-					} else {
-						brlist->next = rlist->next;
-						free(rlist);
-						rlist = brlist->next;
-					}
+		  if (((rlist->route_data)->pathset_ptr) != NULL) {
+		    pathbuf = rlist->route_data;
+		    free(pathbuf->pathset_ptr);
+		    pathbuf->pathset_ptr = NULL;
+		  }
+		  free(rlist->route_data);
+		  if (brlist == NULL) {
+		    node->rinfo = rlist->next;
+		    free(rlist);
+		    rlist = node->rinfo;
+		  } else {
+		    brlist->next = rlist->next;
+		    free(rlist);
+		    rlist = brlist->next;
+		  }
 #ifdef RTHISTORY
-					}
+	        }
 #endif
-					prefixes--;
-					rcode = 0;
-				} else {
-					brlist = rlist;
-					rlist = rlist->next;
-				}
-				/* Back Track for Removing node */
-				if (node->rinfo == NULL) {
-					node->masklen = 0;
-					routes--;
-					while(1) {
-						bnode = node;
-						node = node->upleaf;
-						if ((bnode->r_leaf != NULL) && 
-						    (bnode->l_leaf != NULL)) {
-							break;
-						}
-						if (bnode == node->r_leaf) {
-						  if (bnode->r_leaf == NULL) {
-						    node->r_leaf = bnode->l_leaf;
-						  } else {
-						    node->r_leaf = bnode->r_leaf;
-						  }
-						  if (node->r_leaf != NULL) {
-						    node->r_leaf->upleaf = node;
-						  }
-						} else {
-						  if (bnode->r_leaf == NULL) {
-						    node->l_leaf = bnode->l_leaf;
-						  } else {
-						    node->l_leaf = bnode->r_leaf;
-						  }
-						  if (node->l_leaf != NULL) {
-						    node->l_leaf->upleaf = node;
-						  }
-						}
+	        prefixes--;
+	        rcode = 0;
+	      } else {
+	        brlist = rlist;
+	        rlist = rlist->next;
+	      }
+	      /* Back Track for Removing node */
+	      if (node->rinfo == NULL) {
+	        node->masklen = 0;
+	        routes--;
+	        while(1) {
+		  bnode = node;
+		  node = node->upleaf;
+		  if ((bnode->r_leaf != NULL) && (bnode->l_leaf != NULL)) {
+		    break;
+		  }
+		  if (bnode == node->r_leaf) {
+		    if (bnode->r_leaf == NULL) {
+		      node->r_leaf = bnode->l_leaf;
+		    } else {
+		      node->r_leaf = bnode->r_leaf;
+		    }
+		    if (node->r_leaf != NULL) {
+		      node->r_leaf->upleaf = node;
+		    }
+		  } else {
+		    if (bnode->r_leaf == NULL) {
+		      node->l_leaf = bnode->l_leaf;
+		    } else {
+		      node->l_leaf = bnode->r_leaf;
+		    }
+		    if (node->l_leaf != NULL) {
+		      node->l_leaf->upleaf = node;
+		    }
+		  }
 #ifdef DEBUG
-						errno = 0;
-	printf("DEBUG: RADIX: WITHDRAW: Freeing Node = %08X/%d CB=%d\n",
+		  errno = 0;
+	          printf("DEBUG: RADIX: WITHDRAW: Freeing Node = %08X/%d CB=%d\n",
 			ntohl(bnode->key), bnode->masklen, bnode->check_bit);
 #endif
-						free(bnode);
-						nodes--;
+		  free(bnode);
+		  nodes--;
 #ifdef DEBUG
-	if (errno != 0) perror("DEBUG: RADIX: FreeError: ");
+		  if (errno != 0) perror("DEBUG: RADIX: FreeError: ");
 #endif
-						if (node->upleaf == NULL) break;
-						if (node->rinfo != NULL)  break;
-					}
-					break;
-				}
-			}
-			break;
-		}
-		if (0x80000000 & (ntohl(prefix) << (node->check_bit))) {
-			node = node->r_leaf;
-		} else {
-			node = node->l_leaf;
-		}
-		if (node == NULL) {
-			rcode = 1;
-			break;
-		}
-	}
+		  if (node->upleaf == NULL) break;
+		  if (node->rinfo != NULL)  break;
+	        }
+	        break;
+	      }
+	    }
+	    break;
+	  }
+	  if (0x80000000 & (ntohl(prefix) << (node->check_bit))) {
+	    node = node->r_leaf;
+	  } else {
+	    node = node->l_leaf;
+	  }
+	  if (node == NULL) {
+	    rcode = 1;
+	    break;
+	  }
+        }
 	return(rcode);
 }
 
@@ -522,6 +535,7 @@ u_long delete_peer_route(RADIX_T **cnode, int pn)
 	RADIX_T		*node, *bnode;
 	R_LIST		*rlist, *brlist;
 	u_long		bufnodes;
+	ROUTE_INFO	*pathbuf;
 
 	node = *cnode;
 	rlist = node->rinfo;
@@ -537,6 +551,11 @@ u_long delete_peer_route(RADIX_T **cnode, int pn)
 	printf("                      Neighbor = %08X\n",
 		ntohl(rlist->route_data->source));
 #endif
+			if (((rlist->route_data)->pathset_ptr) != NULL) {
+				pathbuf = rlist->route_data;
+				free(pathbuf->pathset_ptr);
+				pathbuf->pathset_ptr = NULL;
+			}
 			free(rlist->route_data);
 			if (brlist == NULL) {
 				node->rinfo = rlist->next;
@@ -762,37 +781,37 @@ RADIX_T *clear_rt_history(RADIX_T *curnode, int *upcode)
 		printf("DEBUG: RTHISTCLEAN: Goto Most Left Leaf\n");
 #endif
 		while(bnode->l_leaf != NULL) {
-			bnode = bnode->l_leaf;
+		    bnode = bnode->l_leaf;
 #ifdef DEBUG
-			printf("DEBUG: RTHISTCLEAN:   Move to Node = %08X\n",bnode);
+		    printf("DEBUG: RTHISTCLEAN:   Move to Node = %08X\n",bnode);
 #endif
 		}
 		if (bnode->rinfo != NULL) {
 #ifdef DEBUG
-			printf("DEBUG: RTHISTCLEAN:   Found First Route Data\n");
+		    printf("DEBUG: RTHISTCLEAN:   Found First Route Data\n");
 #endif
-			/** Cleaning Node **/
-			rlist = bnode->rinfo;
-			while(rlist != NULL) {
-				rinfo = rlist->route_data;
-				if (rinfo->active == false) {
-					withdraw_route(check_id1topn(rinfo->source),
+		    /** Cleaning Node **/
+		    rlist = bnode->rinfo;
+		    while(rlist != NULL) {
+			rinfo = rlist->route_data;
+			if (rinfo->active == false) {
+			    withdraw_route(check_id1topn(rinfo->source),
 						rinfo->prefix, 
 						rinfo->length,
 						DELETE);
-					rlist = bnode->rinfo;
-				} else {
-					while(rinfo->history != NULL) {
-						histnext = rinfo->history;
-						rinfo->history = histnext->next;
-						histnext->before = NULL;
-						free(histnext);
-					}
-					rinfo->histend = NULL;
-					rlist = rlist->next;
-				}
-			}
-			return(bnode);
+			    rlist = bnode->rinfo;
+		    	} else {
+			    while(rinfo->history != NULL) {
+			        histnext = rinfo->history;
+			        rinfo->history = histnext->next;
+			        histnext->before = NULL;
+			        free(histnext);
+			    }
+			    rinfo->histend = NULL;
+			    rlist = rlist->next;
+		        }
+		    }
+		    return(bnode);
 		}
 	} else {
 		/* Show Current Node */
@@ -875,7 +894,14 @@ void rthisttimerclear()
 			 " ==> Another job is working...\n", OFF);
 		rtclear_id = common_entry_timer(&rthistcleartime, 0,
 							 rthisttimerclear);
-		if (rtclear_id == -1) exit(1);
+		if (rtclear_id == -1) {
+			pr_log(VLOG_GLOB, LOG_ALL,
+				"Timer Set Error : rthisttimerclear - 1\n", ON);
+			notify_systemnotify("REBOOT",
+				"Timer Set Error : rthisttimerclear - 1");
+			finish(PROC_REBOOT);
+			return;
+		}
 		return;
 	}
 
@@ -885,7 +911,14 @@ void rthisttimerclear()
 	if (rthistcleartime.tv_sec != 0) {
 		rtclear_id = common_entry_timer(&rthistcleartime, 0,
 							 rthisttimerclear);
-		if (rtclear_id == -1) exit(1);
+		if (rtclear_id == -1) {
+			pr_log(VLOG_GLOB, LOG_ALL,
+				"Timer Set Error : rthisttimerclear - 1\n", ON);
+			notify_systemnotify("REBOOT",
+				"Timer Set Error : rthisttimerclear - 1");
+			finish(PROC_REBOOT);
+			return;
+		}
 	}
 }
 
