@@ -4,7 +4,7 @@
 /*                                                              */
 /* Routeing Information Update Routine                          */
 /****************************************************************/
-/* route.h,v 1.5 2001/04/19 11:42:57 kuniaki Exp */
+/* route.h,v 1.7 2001/09/16 08:56:13 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -61,7 +61,7 @@
 #define AS_CONFED_SET		3
 #define AS_CONFED_SEQUENCE	4
 
-#define PATHSETSIZE             256
+#define PATHSETSIZE             512
 #define MAXCLUSTERLIST		96
 #define MAXCOMMUNITYLIST	96
 
@@ -115,7 +115,8 @@ typedef struct {
 	char		partial;
 	char		origin_type;
 	char		path_type;
-	char		path_set[PATHSETSIZE];
+	char		*pathset_ptr;
+	/* char		path_set[PATHSETSIZE]; */
 	net_ulong	next_hop;
 	u_long		metric;
 	u_long		local_perf;
