@@ -4,7 +4,7 @@
 /*                                                              */
 /* Routeing Information Update Routine                          */
 /****************************************************************/
-/* update.c,v 1.14 2001/04/19 11:42:58 kuniaki Exp */
+/* update.c,v 1.20 2001/09/19 10:27:54 kuniaki Exp */
 
 /*
  * Copyright (c) 1998-2000 Internet Initiative Japan Inc.
@@ -137,7 +137,10 @@ void pr_route(ROUTE_INFO *routep, int pn)
 	strcat(lbuf, lbuf2);
 	sprintf(lbuf2, "%7d ", timediff);
 	strcat(lbuf, lbuf2);
-	strcat(lbuf, routep->path_set);
+	/* strcat(lbuf, routep->path_set); */
+	if (routep->pathset_ptr != NULL) {
+		strcat(lbuf, routep->pathset_ptr);
+	}
 	if (ATRFL_ISSET(routep->attribute_flag, SET_ORIGIN)) {
 		switch(routep->origin_type) {
 		case ORIGIN_IGP:
@@ -162,10 +165,12 @@ void pr_update(int pn)
 	struct in_addr 	inaddr;
 	int		length, le1, le2, cnt, ptmp;
 	int		attr_len, plen, ret;
-	int		attr_octs, type_code;
+	u_int		attr_octs, type_code;
 	ROUTE_INFO	getroute, prroute;
 	char		logstr[128];
 	prefix_t	outpre;
+	char		pathbuf[4096];
+	int		pathbuflen;
 
 	static char	*path_attr[] = {
 			/* 0 */ "UNKNOWN",
@@ -299,11 +304,11 @@ void pr_update(int pn)
 			if (*ptr & 0x10) {
 				attr_len = 2;
 				pr_log(VLOG_UPAT, pn, 
-					"    Attribute Length Enhanced\n");
+					"    Attribute Length Enhanced\n", OFF);
 			} else {
 				attr_len = 1;
 				pr_log(VLOG_UPAT, pn, 
-					"    Attribute Length Normal\n");
+					"    Attribute Length Normal\n", OFF);
 			}
 
 			if (type_code > ATTR_MAX) {
@@ -317,10 +322,10 @@ void pr_update(int pn)
 			ptr += 2;
 	
 			if (attr_len == 1) {
-				attr_octs = (int)*ptr;
+				attr_octs = (u_char)*ptr;
 				ptr++;
 			} else {
-				attr_octs = (int)buf2ushort(ptr);
+				attr_octs = (u_short)buf2ushort(ptr);
 				ptr += 2;
 			}
 			sprintf(logstr,
@@ -360,7 +365,8 @@ void pr_update(int pn)
 				ATRFL_SET(SET_PATHTYPE,
 						getroute.attribute_flag);
 				ptmp = 0;
-				getroute.path_set[0] = '\0';
+				/* getroute.path_set[0] = '\0'; */
+				pathbuf[0] = '\0';
 				while(ptmp < attr_octs) {
 				  strcpy(logstr, "    PATH Segment Type = ");
 				  getroute.path_type = *(ptr+ptmp);
@@ -391,29 +397,47 @@ void pr_update(int pn)
 				  switch(getroute.path_type) {
 				    case AS_CONFED_SET:
 				    case AS_CONFED_SEQUENCE:
-					strcat(getroute.path_set, "(");
+					/* strcat(getroute.path_set, "("); */
+					strcat(pathbuf, "(");
 					break;
 				  }
 				  cp = ptr + ptmp;
 				  for(cnt = 0; cnt < length; cnt++, cp += 2){
 					sprintf(cbuf, "%d ",
 						(int)buf2ushort(cp));
-					strcat(getroute.path_set, cbuf);
+					/* strcat(getroute.path_set, cbuf); */
+					strcat(pathbuf, cbuf);
 				  }
 				  switch(getroute.path_type) {
 				    case AS_CONFED_SET:
 				    case AS_CONFED_SEQUENCE:
-					cnt = strlen(getroute.path_set);
+					/* cnt = strlen(getroute.path_set); */
+					cnt = strlen(pathbuf);
 					if (cnt > 0) {
-					  getroute.path_set[cnt-1] = '\0';
+					  /* getroute.path_set[cnt-1] = '\0'; */
+					  pathbuf[cnt-1] = '\0';
 					}
-					strcat(getroute.path_set, ") ");
+					/* strcat(getroute.path_set, ") "); */
+					strcat(pathbuf, ") ");
 					break;
 				  }
 				  ptmp += length*2;
 				}
-				sprintf(logstr, "    AS_Path = %s\n",
-							getroute.path_set);
+				
+				/* Add 16 Sep 2001 Dynamic Path set */
+				if (strlen(pathbuf) == 0) {
+					getroute.pathset_ptr = NULL;
+					sprintf(logstr, "    AS_Path = N/A\n");
+				} else {
+					getroute.pathset_ptr = &pathbuf[0];
+					strcpy(getroute.pathset_ptr, pathbuf);
+					/*
+					sprintf(logstr, "    AS_Path = %s\n",
+								getroute.path_set);
+					*/
+					sprintf(logstr, "    AS_Path = %s\n",
+								getroute.pathset_ptr);
+				}
 				pr_log(VLOG_UPAT, pn, logstr, OFF);
 				ptr += attr_octs;
 				break;
@@ -569,6 +593,22 @@ void pr_update(int pn)
 
 		memmove((u_char *)&prroute.prefix, (u_char *)(ptr+pt2+1), cnt);
 
+		if (prroute.pathset_ptr != NULL) {
+		    pathbuflen = strlen(getroute.pathset_ptr);
+		    if (pathbuflen != 0) {
+			prroute.pathset_ptr = (char *)malloc(pathbuflen+1);
+			if (prroute.pathset_ptr == NULL) {
+			    pr_log(VLOG_GLOB, LOG_ALL, 
+			        "pr_update: pathset_ptr: Malloc error\n", ON);
+			    notify_systemnotify("REBOOT",
+			    			"Malloc Error : pr_update");
+			    finish(PROC_REBOOT);
+			    return;
+			}
+		    }
+		    strcpy(prroute.pathset_ptr, getroute.pathset_ptr);
+		}
+
 		pr_route(&prroute, pn);
 		bgpinfo[pn].prefixcnt++;
 		pt2 += cnt+1;
